#include "ruby.h"
#include "libnet.h"
#include <stdio.h>
#include <string.h>

#define LIBNET_LABEL_SIZE 64
VALUE cLibnet;

static VALUE rnet_write(VALUE self) {
    int ret;
    libnet_t *l;
    Data_Get_Struct(self, libnet_t, l);
    
    ret = libnet_write(l);

    if (ret == -1) {
        rb_raise (rb_eRuntimeError, libnet_geterror(l));
    }

    return Qnil;
}

static VALUE rnet_write_raw(VALUE self, VALUE pkt, VALUE size) {
	int ret;
	u_int32_t _size;
	u_int8_t * _pkt;
	
    	libnet_t *l;
    	Data_Get_Struct(self, libnet_t, l);
	
	_size = (u_int32_t)NUM2INT(size);
	_pkt = (u_int8_t *)RSTRING(pkt)->ptr;
	
	ret = libnet_write_raw_ipv4(l, _pkt, _size);
	
	if (ret == -1) {
        	rb_raise (rb_eRuntimeError, libnet_geterror(l));
    	}

    return Qnil;
}

static VALUE rnet_icmp_echo(VALUE self, VALUE type, VALUE code, VALUE chksum, VALUE id, VALUE seqnum, VALUE payload, VALUE payloadsize) {
    libnet_ptag_t t;
    libnet_t *l;
    u_int8_t _type, _code;
    u_int16_t _chksum, _id, _seqnum;
    u_int32_t _payloadsize, len;
    u_int8_t * _payload;
    Data_Get_Struct(self, libnet_t, l);
    
    if (l == NULL) {
        rb_raise(rb_eRuntimeError, "no handle?");
    }
	
    _type = (u_int8_t)NUM2INT(type);
    _code = (u_int8_t)NUM2INT(code);
    _chksum = (u_int16_t)NUM2INT(chksum);
    _id = (u_int16_t)NUM2INT(id);
    _seqnum = (u_int16_t)NUM2INT(seqnum);
    _payloadsize = (u_int32_t)NUM2INT(payloadsize);
    
    if (RSTRING(payload)->len == 0){
    	_payload = NULL;
    } else {
       _payload = ( u_int8_t *)RSTRING(payload)->ptr;
    }   	

    t = libnet_build_icmpv4_echo(
            _type,                            /* type */
            _code,                            /* code */
            _chksum,                          /* checksum */
            _id,                              /* id */
            _seqnum,                          /* sequence number */
            _payload,                         /* payload */
            _payloadsize,                     /* payload size */
            l,                                /* libnet handle */
            0);

    if (t == -1) {
        rb_raise (rb_eRuntimeError, libnet_geterror(l));
    }

    return Qnil;
}

static VALUE rnet_ip (VALUE self, VALUE len, VALUE tos, VALUE id, VALUE frag, VALUE ttl, VALUE proto, VALUE chksum, VALUE src, VALUE
dst, VALUE payload, VALUE payloadsize) {
    libnet_ptag_t t;
    libnet_t *l;
    u_int16_t _len, _id, _frag, _chksum;
    u_int8_t _tos, _ttl, _proto;
    u_int32_t src_ip, dst_ip;
    u_int32_t _payloadsize;
    u_int8_t * _payload;    
    Data_Get_Struct(self, libnet_t, l);

    if (l == NULL) {
        rb_raise(rb_eRuntimeError, "no handle?");
    }
     
    _len= (u_int16_t)NUM2INT(len);
    _tos= (u_int8_t)NUM2INT(tos);
    _id = (u_int16_t)NUM2INT(id);
    _frag = (u_int16_t)NUM2INT(frag);
    _ttl= (u_int8_t)NUM2INT(ttl);
    _proto= (u_int8_t)NUM2INT(proto);
    _chksum = (u_int16_t)NUM2INT(chksum);
    src_ip = libnet_name2addr4(l, RSTRING(src)->ptr, LIBNET_RESOLVE);
    dst_ip = libnet_name2addr4(l, RSTRING(dst)->ptr, LIBNET_RESOLVE);
    
    if ((dst_ip == -1) || (src_ip == -1)){
    	rb_raise(rb_eRuntimeError, "bad IP address");
	return Qnil;
    }
    
    
    _payloadsize = (u_int32_t)NUM2INT(payloadsize);
    
   
    if (RSTRING(payload)->len == 0){
    	_payload = NULL;
    } else {
       _payload = ( u_int8_t *)RSTRING(payload)->ptr;
    }   	

   

    t = libnet_build_ipv4(
            _len, 			/* length */
            _tos,                       /* TOS */
            _id,                        /* IP ID */
            _frag,                      /* IP Frag */
            _ttl,                       /* TTL */
            _proto,                     /* protocol */
            _chksum,                    /* checksum */
            src_ip,                     /* source IP */
            dst_ip,                     /* destination IP */
            _payload,                   /* payload */
            _payloadsize,               /* payload size */
            l,                          /* libnet handle */
            0);

    if (t == -1) {
        rb_raise (rb_eRuntimeError, libnet_geterror(l));
    }

    return Qnil;
}

static VALUE rnet_tcp (VALUE self, VALUE sp, VALUE dp, VALUE seqnum, VALUE ack, VALUE control, VALUE win, VALUE chksum, VALUE urg, VALUE len,VALUE payload, VALUE payloadsize) {
    libnet_ptag_t t;
    libnet_t *l;
    u_int16_t _len, _dp, _sp, _chksum, _win, _urg;
    u_int8_t _control;
    u_int32_t _seqnum, _ack;
    u_int32_t _payloadsize;
    u_int8_t * _payload;
    Data_Get_Struct(self, libnet_t, l);

    if (l == NULL) {
        rb_raise(rb_eRuntimeError, "no handle?");
    }
    _sp= (u_int16_t)NUM2INT(sp);
    _dp= (u_int16_t)NUM2INT(dp);
    _seqnum = (u_int32_t)NUM2INT(seqnum);
    _ack = (u_int32_t)NUM2INT(ack);	
    _control= (u_int8_t)NUM2INT(control);
    _win = (u_int16_t)NUM2INT(win);	
    _chksum = (u_int16_t)NUM2INT(chksum);
    _urg = (u_int16_t)NUM2INT(urg);
    _len= (u_int16_t)NUM2INT(len);

    _payloadsize = (u_int32_t)NUM2INT(payloadsize);
    
    if (RSTRING(payload)->len == 0){
    	_payload = NULL;
    } else {
       _payload = ( u_int8_t *)RSTRING(payload)->ptr;
    }   	

   

    t = libnet_build_tcp(
            _sp, 			/* source port */
            _dp, 			/* destination port */
            _seqnum, 			/* sequence number */
            _ack, 			/* acknowledgement number */
            _control, 			/* control flags */
            _win, 			/* window size */
            _chksum, 			/* checksum (0 for libnet to autofill) */
            _urg, 			/* urgent pointer */
	    _len, 			/* total length of the TCP packet (for checksum calculation) */
            _payload,                   /* payload */
            _payloadsize,               /* payload size */
            l,                          /* libnet handle */
            0);

    if (t == -1) {
        rb_raise (rb_eRuntimeError, libnet_geterror(l));
    }

    return Qnil;
}

static VALUE rnet_udp (VALUE self, VALUE sp, VALUE dp, VALUE chksum, VALUE len,VALUE payload, VALUE payloadsize) {
    libnet_ptag_t t;
    libnet_t *l;
    u_int16_t _len, _dp, _sp, _chksum;
    u_int32_t _payloadsize;
    u_int8_t * _payload;
    Data_Get_Struct(self, libnet_t, l);

    if (l == NULL) {
        rb_raise(rb_eRuntimeError, "no handle?");
    }
    _sp= (u_int16_t)NUM2INT(sp);
    _dp= (u_int16_t)NUM2INT(dp);
    _chksum = (u_int16_t)NUM2INT(chksum);
    _len= (u_int16_t)NUM2INT(len);

    _payloadsize = (u_int32_t)NUM2INT(payloadsize);
    
    if (RSTRING(payload)->len == 0){
    	_payload = NULL;
    } else {
       _payload = ( u_int8_t *)RSTRING(payload)->ptr;
    }   	

   

    t = libnet_build_udp(
            _sp, 			/* source port */
            _dp, 			/* destination port */
            _len, 			/* total length of the UDP packet */
            _chksum,			/* checksum (0 for libnet to autofill) */
            _payload,                   /* payload */
            _payloadsize,               /* payload size */
            l,                          /* libnet handle */
            0);

    if (t == -1) {
        rb_raise (rb_eRuntimeError, libnet_geterror(l));
    }

    return Qnil;
}

static VALUE rnet_eth (VALUE self, VALUE src, VALUE dst, VALUE ptype, VALUE payload, VALUE payloadsize) {
    /* both src and dst are handled as strings */

    libnet_ptag_t t;
    libnet_t *l;
    
    u_int16_t _ptype;
    u_int32_t _payloadsize;
    u_int8_t * _payload;
    u_int8_t * _dst;
    u_int8_t * _src;
    
    _ptype= (u_int16_t)NUM2INT(ptype);
    _payloadsize = (u_int32_t)NUM2INT(payloadsize);
        
    if ((RSTRING(src)->len) != 6 || (RSTRING(dst)->len != 6)){
    	rb_raise(rb_eRuntimeError, "bad MAC address");
	return Qnil;
    }
    
    _dst = (u_int8_t *)RSTRING(dst)->ptr;
    _src = (u_int8_t *)RSTRING(src)->ptr;
    
    Data_Get_Struct(self, libnet_t, l);
    if (l == NULL) {
        rb_raise(rb_eRuntimeError, "no handle?");
    }

    if (RSTRING(payload)->len == 0){
    	_payload = NULL;
    } else {
       _payload = ( u_int8_t *)RSTRING(payload)->ptr;
    }   	
    
    t = libnet_build_ethernet(
            _dst,                                   /* ethernet destination */
            _src,                                   /* ethernet source */
            _ptype,                                  /* protocol type */
            _payload,                               /* payload */
            _payloadsize,                           /* payload size */
            l,                                      /* libnet handle */
            0);                                     /* libnet id */

    if (t == -1) {
        rb_raise (rb_eRuntimeError, libnet_geterror(l));
    }

    return Qnil;
}

static VALUE rnet_arp (VALUE self, VALUE op, VALUE sha, VALUE spa, VALUE tha, VALUE tpa) {
    /* addresses are handled as strings */

    libnet_ptag_t t;
    libnet_t *l;
    
    u_int16_t _op;
    u_int32_t src_ip, dst_ip;
    u_int8_t * _sha;
    u_int8_t * _spa;
    u_int8_t * _tha;
    u_int8_t * _tpa;
    
    _op= (u_int16_t)NUM2INT(op);
   
    src_ip = libnet_name2addr4(l, RSTRING(spa)->ptr, LIBNET_RESOLVE);
    dst_ip = libnet_name2addr4(l, RSTRING(tpa)->ptr, LIBNET_RESOLVE);
    
    if ((dst_ip == -1) || (src_ip == -1)){
    	rb_raise(rb_eRuntimeError, "bad IP address");
	return Qnil;
    }
    
    _spa = (u_int8_t *) &src_ip;
    _tpa = (u_int8_t *) &dst_ip;
        
    if ((RSTRING(sha)->len) != 6 || (RSTRING(tha)->len != 6)){
    	rb_raise(rb_eRuntimeError, "bad MAC address");
	return Qnil;
    }
    
    _sha = (u_int8_t *)RSTRING(sha)->ptr;
    _tha = (u_int8_t *)RSTRING(tha)->ptr;
    
    Data_Get_Struct(self, libnet_t, l);
    if (l == NULL) {
        rb_raise(rb_eRuntimeError, "no handle?");
    }

    t = libnet_autobuild_arp(
            _op,                                   /* operation type */
            _sha,				   /* sender's hardware address */
            _spa,				   /* sender's protocol address */
            _tha,				   /* target hardware address */
            _tpa,				   /* targer protocol address */
            l);					   /* libnet handle */

    if (t == -1) {
        rb_raise (rb_eRuntimeError, libnet_geterror(l));
    }

    return Qnil;
}

static void rnet_mark (libnet_t *l) {}
static void rnet_free (libnet_t *l) {
    libnet_destroy(l);
}

static VALUE rnet_init(VALUE self, VALUE unit) {
    rb_iv_set(self, "@handle", unit);
    return self;
}

VALUE rnet_getip(VALUE self){
	
	libnet_t *l;
	u_int32_t ip;
	char *ip_txt;	
	VALUE result;

	Data_Get_Struct(self, libnet_t, l);
    	
	if (l == NULL) {
        	rb_raise(rb_eRuntimeError, "no handle?");
    	}

	if((ip = libnet_get_ipaddr4 (l)) == -1){
		rb_raise(rb_eRuntimeError, "Unable to get IP address");
		return Qnil;
    	}
	
	ip_txt = libnet_addr2name4(ip, LIBNET_DONT_RESOLVE);
	result = rb_str_new2(ip_txt);
	return result; 
}

VALUE rnet_getmac(VALUE self){
	libnet_t *l;
	VALUE result;
	struct libnet_ether_addr *ethaddr;
	char mac[18];
	int i;
	
	Data_Get_Struct(self, libnet_t, l);
    	
	if (l == NULL) {
        	rb_raise(rb_eRuntimeError, "no handle?");
    	}

	if ((ethaddr = libnet_get_hwaddr(l)) == NULL)
        {
            	rb_raise(rb_eRuntimeError, "Unable to get MAC address");
		return Qnil;
        }

        snprintf(mac, 18, "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\0", ethaddr->ether_addr_octet[0], ethaddr->ether_addr_octet[1],ethaddr->ether_addr_octet[2],ethaddr->ether_addr_octet[3],ethaddr->ether_addr_octet[4],ethaddr->ether_addr_octet[5]);
	result = rb_str_new2(mac);
	return result; 

}

VALUE rnet_getdevice(VALUE self){
	libnet_t *l;
	VALUE result;
	char *device;
	char nulldevice[] = "";
	Data_Get_Struct(self, libnet_t, l);
    	
	if (l == NULL) {
        	rb_raise(rb_eRuntimeError, "no handle?");
    	}

	if ((device = (char *)libnet_getdevice(l)) == NULL){
		device = nulldevice;	
	}
	result = rb_str_new2(device);
	return result; 

}

VALUE rnet_new(VALUE class, VALUE unit, VALUE type, VALUE device) {
    char errbuf[LIBNET_ERRBUF_SIZE];
    VALUE argv[1];
    int _type;
    char * _device;
    
    if (RSTRING(device)->len == 0){
	_device = NULL;
    } else {
	_device = (char *)RSTRING(device)->ptr;
    } 
    _type = NUM2INT(type);
   	
    libnet_t *l = libnet_init(_type, _device, errbuf); 

    if (l == NULL) {
        rb_raise (rb_eRuntimeError, errbuf);
    }

    VALUE tdata = Data_Wrap_Struct(class, 0, rnet_free, l);
    argv[0] = unit;
    rb_obj_call_init(tdata, 1, argv);
    return tdata;
}

void Init_libnet4r() {
    cLibnet = rb_define_class("Libnet", rb_cObject);
    rb_define_singleton_method(cLibnet, "new", rnet_new, 3);
    rb_define_method(cLibnet, "initialize", rnet_init, 1);
    rb_define_method(cLibnet, "write", rnet_write, 0);
    rb_define_method(cLibnet, "write_raw", rnet_write_raw, 2);
    rb_define_method(cLibnet, "icmp", rnet_icmp_echo, 7);
    rb_define_method(cLibnet, "ip", rnet_ip, 11);
    rb_define_method(cLibnet, "eth", rnet_eth, 5);
    rb_define_method(cLibnet, "arp", rnet_arp, 5);
    rb_define_method(cLibnet, "tcp", rnet_tcp, 11);
    rb_define_method(cLibnet, "udp", rnet_udp, 6);
    rb_define_method(cLibnet, "getip", rnet_getip, 0);
    rb_define_method(cLibnet, "getmac", rnet_getmac, 0);
    rb_define_method(cLibnet, "getdevice", rnet_getdevice, 0);
}
