require 'fox14'
require 'rubyforger.rb'

include Fox

init("lo") # init rubyforger. Add a preference menu to let the user choose the interface 

# TODO  - make everything a little prettier (especially the Properties dialog, that could include a tree like diagramm) - done
#	- add a timeout to the send/receive thread	- done
#	- add a Capture button : input a tcpdump rule, and capture packets	- done
#	- add a preference menu	- done
#	- add a better way to navigate thru the packets, and give them names (maybe a little tree like stuff left) - done
#	- add a way to input a pcap file
#	- catch exceptions :)
#	- add a contextual menu on right click 
#	- make pretty icons
#	- add a code interpreter, to acces directly rubyforger functions 
#	- verify the input when changing layer parameters

# global vars 

$count = 0		# number of packets ( used to give a default name to the packets )
$currentedit = nil	# The packet currently being edited
$filename = nil		# the file being edited		
$layers = []		# the 5 layers, used to draw the packets 
$TIMEOUT = 1		# the timeout for the receive threads

##########################
##	  GUI		##
##########################

$app = FXApp.new
$main = FXMainWindow.new($app,'VisualForge')

# open the image files
$layericon = []
$protoicon = {}
# for the layers
1.upto(5) do |layer|
	$layericon[layer] = FXJPGIcon.new($main.getApp(), File.open("images/layer#{layer}.jpg", "rb").read)
end
# and for each protocol
["ETH","ARP","Raw data","IP","TCP","UDP","ICMP"].each do |proto|
	$protoicon[proto] = FXJPGIcon.new($main.getApp(), File.open("images/#{proto}.jpg", "rb").read)
end

# and the icons
$sendIcon = FXPNGIcon.new($main.getApp(), File.open("images/send.png", "rb").read)
$sendreceiveIcon = FXPNGIcon.new($main.getApp(), File.open("images/sendreceive.png", "rb").read)
$codeIcon = FXPNGIcon.new($main.getApp(), File.open("images/code.png", "rb").read)
$checkIcon = FXPNGIcon.new($main.getApp(), File.open("images/check.png", "rb").read)
$addIcon = FXPNGIcon.new($main.getApp(), File.open("images/add.png", "rb").read)
$removeIcon = FXPNGIcon.new($main.getApp(), File.open("images/remove.png", "rb").read)
$receiveIcon = FXPNGIcon.new($main.getApp(), File.open("images/receive.png", "rb").read)
$renameIcon = FXPNGIcon.new($main.getApp(), File.open("images/rename.png", "rb").read)

# menu
menubar = FXMenuBar.new($main, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
filemenu = FXMenuPane.new($main)
FXMenuCommand.new(filemenu, "&New\tCtl-N\tNew File.", nil).connect(SEL_COMMAND){ newFile }
FXMenuCommand.new(filemenu, "&Open\tCtl-O\tOpen File.", nil).connect(SEL_COMMAND){ openFile }
FXMenuCommand.new(filemenu, "&Save\tCtl-S\tSave File.", nil).connect(SEL_COMMAND){ saveFile }
FXMenuCommand.new(filemenu, "&Save As\tCtl-A\tSave File As...", nil).connect(SEL_COMMAND){ saveFileAs }
FXMenuCommand.new(filemenu, "&Quit\tCtl-Q\tQuit the application.", nil, $main.getApp, FXApp::ID_QUIT)
FXMenuTitle.new(menubar, "&File", nil, filemenu)
settingmenu = FXMenuPane.new($main)
FXMenuTitle.new(menubar, "&Settings", nil, settingmenu)
FXMenuCommand.new(settingmenu, "&Preferences\tCtl-P\tChange Settings.", nil).connect(SEL_COMMAND){ preferences }
aboutmenu = FXMenuPane.new($main)
FXMenuTitle.new(menubar, "&About", nil, aboutmenu)
FXMenuCommand.new(aboutmenu, "&About\tCtl-A\tAbout.", nil).connect(SEL_COMMAND){ showDlg("About","Visualforge, a GUI packet crafter.\nVersion 0.02, September 2006\n0xDEAD - castorpilot@yahoo.com\nReleased under the GPL license") }
# status bar
FXStatusBar.new($main ,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER)

# main window
contents = FXHorizontalFrame.new($main,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)

############################
# LEFT pane for the packet #
############################
pktFrame = FXVerticalFrame.new(contents,FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10)

###########################
# create a button Frame   #
###########################
FXHorizontalSeparator.new(pktFrame, SEPARATOR_GROOVE|LAYOUT_FILL_X)
buttonFrame=FXHorizontalFrame.new(pktFrame, LAYOUT_SIDE_TOP|FRAME_NONE|PACK_UNIFORM_WIDTH)

# create a "send button"
FXButton.new(buttonFrame,"", $sendIcon){ |button|
	button.connect(SEL_COMMAND){
		pkt = buildpkt
		sendpkt(pkt) if pkt != false
	}
	button.tipText = "Send"
}

# create a Send/receive button

FXButton.new(buttonFrame,"",$sendreceiveIcon){|button|
	button.connect(SEL_COMMAND){
		pkt = buildpkt
		if pkt != false
			t = Thread.new { rpkt = sr(pkt)
				 addPkt(rpkt)
			}
			Thread.new{
				sleep($TIMEOUT)
				Thread.kill(t)
			}
		end
	}
	button.tipText = "Send/Receive"
}

FXButton.new(buttonFrame,"",$receiveIcon){|button|
	button.connect(SEL_COMMAND){ 
			receive
	}
	button.tipText = "Receive"
}

# create a Check button
FXButton.new(buttonFrame,"",$checkIcon){|button|
	button.connect(SEL_COMMAND){
		code = checkpkt
		if code
			showDlg("Check","The packet seems valid")
		else
			showError("Invalid packet")
		end
	}
	button.tipText = "Check Packet"
}

# create a Show Ruby code button
FXButton.new(buttonFrame,"",$codeIcon){|button|
	button.connect(SEL_COMMAND){ 
		showCode(pktCode($currentedit.data))
	}
	button.tipText = "Show Ruby Code"
}	
		
# create a "New" button
FXButton.new(buttonFrame,"",$addIcon){|button|
	button.connect(SEL_COMMAND){
		addPkt(NullPkt.new)
	}	
	button.tipText = "Add New Packet"
}

# create a rename button
FXButton.new(buttonFrame,"",$renameIcon){ |button|
	button.connect(SEL_COMMAND){
		pktRename()
	}
	button.tipText = "Rename"
}

# create a delete button
FXButton.new(buttonFrame,"",$removeIcon){ |button|
	button.connect(SEL_COMMAND){
		pktDelete()
	}
	button.tipText = "Delete"
}


# Horizontal divider line
FXHorizontalSeparator.new(pktFrame, SEPARATOR_GROOVE|LAYOUT_FILL_X)

# And a frame packet information
nav= FXHorizontalFrame.new(pktFrame, LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_CENTER_X)

$title = FXLabel.new(nav,"",nil,LAYOUT_CENTER_X)

# a splitter for the two tabs
splitter = FXSplitter.new(pktFrame, (LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_REVERSED|SPLITTER_TRACKING))
left = FXVerticalFrame.new(splitter,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)
right = FXVerticalFrame.new(splitter,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y)

##################################
#	The Packet Tree		 #
##################################
$pktTree = FXTreeList.new(left, nil, 0,(LAYOUT_FILL_Y|TREELIST_SHOWS_LINES|LAYOUT_FILL_X|      TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES|TREELIST_EXTENDEDSELECT)) 
$pktTree.connect(SEL_RIGHTBUTTONPRESS) do |sender, sel, item|
# TODO : add a menu
end
$pktTree.connect(SEL_COMMAND) do |sender, sel, item|
	# on click, select the choosen packet
	selectPkt(item)
end

#########################
#    The Packet Page	#
#########################
pktPage = FXPacker.new(right, LAYOUT_FILL_Y|LAYOUT_FILL_X|LAYOUT_CENTER_X|FRAME_THICK|FRAME_RAISED)

# Draw a nice scrolling Frame
scroll=FXScrollWindow.new(pktPage,LAYOUT_FILL_Y|LAYOUT_FILL_X|LAYOUT_CENTER_X, 0, 0,0, 0)
newFrame = FXVerticalFrame.new(scroll,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_CENTER_X, 0, 0, 0, 0, 0, 0, 0, 0)

# For each layer, create a button with an icon
5.downto(1) do |layer|
	$layers[layer] = FXButton.new(newFrame,"",$layericon[layer],nil,0,ICON_ABOVE_TEXT|LAYOUT_CENTER_X)
	$layers[layer].connect(SEL_COMMAND) { clearlayer(layer) }
	$layers[layer].connect(SEL_RIGHTBUTTONPRESS) { configurelayer(layer) }
end


##########################################
#   RIGHT pane for the building blocks   #
##########################################
componentFrame = FXVerticalFrame.new(contents, FRAME_SUNKEN|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10)

# Switcher
tabbook = FXTabBook.new(componentFrame, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)

#  For each layer, create a set of buttons
1.upto(5) do |layer|

	# create a list of the available protocols
	protocols = case layer
		when 1 then ["ETH"]
		when 2 then ["ARP","Raw data"]
		when 3 then ["IP"]
		when 4 then ["TCP","UDP","ICMP","Raw data"]
		when 5 then ["Raw data"]
	end

	# Create a tab that scrolls
	imagesTab = FXTabItem.new(tabbook, "Layer#{layer}", nil,LAYOUT_FILL_Y)
	imagesPage = FXPacker.new(tabbook, LAYOUT_FILL_Y|FRAME_THICK|FRAME_RAISED)
	scroll=FXScrollWindow.new(imagesPage,LAYOUT_FILL_Y|LAYOUT_FILL_X, 0, 0,0, 0)
	newFrame = FXVerticalFrame.new(scroll,LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)

	# Create a button list : one for each protocol
	protocols.each do |proto|
		FXButton.new(newFrame,"",$protoicon[proto],nil,0,ICON_ABOVE_TEXT|LAYOUT_CENTER_X){ |button|
			button.connect(SEL_COMMAND) { updatepkt(layer, proto) }
		}
	end    

end

#######################
## 	Methods      ##
#######################

# update the packet by adding a new protocol
def updatepkt(layer, proto)	
	pkt = case proto
		when "ETH" then EthPkt.new
		when "ARP" then ArpPkt.new
		when "IP" then IpPkt.new
		when "TCP" then TcpPkt.new
		when "UDP" then UdpPkt.new
		when "ICMP" then IcmpPkt.new
		when "Raw data" then ""
	end
	$currentedit.data[layer] = pkt
	updateTree()
	$layers[layer].icon = $protoicon[proto]	
end

def clearlayer(layer)	
	$currentedit.data[layer] = nil	
	updateTree()
	$layers[layer].icon = $layericon[layer]
end

def updateTree
	# delete existing children
	$pktTree.collapseTree($currentedit)
	
	# ugly hack : we can't remove the child by doing a single each loop.
	# Because the iterator is broken if we delete the item it needs to 
	# call a getNext method on.
	tab = []
	$currentedit.each do |child|
		tab.push(child)
	end
	tab.each do |child|
		$pktTree.removeItem(child)
	end	

	# add the new nodes
	$currentedit.data.each do |layer|
		if layer.kind_of? Pkt
			tmp = $pktTree.appendItem($currentedit, getLayerName(layer))
			layer.to_s.each do |line|
				$pktTree.appendItem(tmp, line.chomp) if !(line =~ /#/)
			end
		end
		if layer.instance_of? String
			tmp = $pktTree.appendItem($currentedit, getLayerName(layer))
			$pktTree.appendItem(tmp,layer.gsub(/\W/){ |a|  sprintf("\\x%x",a[0])})
		end
	end
	$pktTree.expandTree($currentedit)
end

def getLayerName(layer)
	return "ETH" if layer.instance_of? EthPkt
	return "ARP" if layer.instance_of? ArpPkt
	return "IP" if layer.instance_of? IpPkt
	return "TCP" if layer.instance_of? TcpPkt
	return "UDP" if layer.instance_of? UdpPkt
	return "ICMP" if layer.instance_of? IcmpPkt
	return "Data" if layer.instance_of? String
end

def buildpkt
	pkt = NullPkt.new
	# find the "lowest level" layer
	lowest = 0
	$currentedit.data.each { |i|
		if i.kind_of? Pkt
			pkt = i.dup
			lowest = $currentedit.data.index i
			break
		end
	}
	
	if (lowest != 1 and lowest != 3)
		showError("This packet cannot be sent") 
		return false
	end

	# check if the packet is valid
	if checkpkt
		# now create the rest of the packet
		(lowest+1).upto(5) do |i|			
			pkt + $currentedit.data[i].dup if $currentedit.data[i] != nil
		end
		return pkt
	else
		showError("Invalid packet")
		return false
	end
end

def sendpkt(pkt)
	pkt.send
end

def sr(pkt)
	return pkt.sr
end

def receive
	# put a textbox to enter the string
	dlg=FXDialogBox.new($main,"Pcap Filter")
	# Bottom buttons
	buttons = FXHorizontalFrame.new(dlg, LAYOUT_SIDE_BOTTOM|FRAME_NONE|LAYOUT_FILL_X,0, 0, 0, 0, 40, 40, 20, 20)
	
	# Accept
	accept = FXButton.new(buttons, "&Accept", nil, dlg, FXDialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
 	accept.setDefault  
    	accept.setFocus   
	
	# Cancel
	FXButton.new(buttons, "&Cancel", nil, dlg, FXDialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
 
	frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE)
	FXLabel.new(frame,"Pcap Filter :")
	data = FXTextField.new(frame,50)
	frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE)
	FXLabel.new(frame,"Size of the captured packets :")
	length = FXTextField.new(frame,6)
	length.text = "1500"
	frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE)
	FXLabel.new(frame,"Number of packets to be captured")
	count = FXTextField.new(frame,6)
	count.text = "1"
	frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE)
	dns = FXCheckButton.new(frame,"DNS lookup")
	dns.setCheck(TRUE)
	exitcode = dlg.execute
	if exitcode != 0
		sniffstring = "-i #$INTERFACE"
		sniffstring << " -n" unless dns.checked?
		if count.text.to_i > 0
			sniffstring << " -c #{count.text.to_i}"
		else
			sniffstring << " -c 1"
		end
		if length.text.to_i > 1500
			sniffstring << " -s #{length.text.to_i}"
		else
			sniffstring << " -s 1500"
		end
		t=Thread.new{
			sniff = Pcaplet.new(sniffstring)
			filter = Pcap::Filter.new(data.text, sniff.capture)
			sniff.add_filter(filter)
			for receivedpacket in sniff
				if filter =~ receivedpacket
					addPkt(pcap2pkt(receivedpacket))
				end
			end
		}
		Thread.new{
			sleep($TIMEOUT)
			Thread.kill(t)
		}
	end
end

def checkpkt
	# check if packet is correct
	if $currentedit.data[2].kind_of? Pkt #if Arp is used, there should not be any upper layer except for Raw data
		3.upto(5) {|i|
			return false if $currentedit.data[i].kind_of? Pkt
		}
	end
	# if IP is not set, no layer 4 is accepted
	if !($currentedit.data[3].kind_of? Pkt)
		4.upto(5) {|i|
			return false if $currentedit.data[i].kind_of? Pkt
		}
	end

	return true
end

def configurelayer(layer)

# if the layer is not empty 
if $currentedit.data[layer] != nil
	
	# Dialog box
	dlg=FXDialogBox.new($main,"Parameters")
	
	# Bottom buttons
	buttons = FXHorizontalFrame.new(dlg, LAYOUT_SIDE_BOTTOM|FRAME_NONE|LAYOUT_FILL_X,0, 0, 0, 0, 40, 40, 20, 20)
	
	# Accept
	accept = FXButton.new(buttons, "&Accept", nil, dlg, FXDialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
 	accept.setDefault  
    	accept.setFocus   
	
	# Cancel
	FXButton.new(buttons, "&Cancel", nil, dlg, FXDialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
 
	if $currentedit.data[layer].kind_of? Pkt
	
		pktinputfield = {}
		# create Input box for each attribute of the packet
		$currentedit.data[layer].attr.each_key do |attr|
        		frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE|PACK_UNIFORM_WIDTH)
			FXLabel.new(frame,attr)
			pktinputfield[attr] = FXTextField.new(frame,10)
			pktinputfield[attr].text = $currentedit.data[layer].attr[attr].to_s		
		end
		
		# lauch dialog
		exitcode = dlg.execute

		# if accept, update the values that have been changed
		if exitcode != 0
			pktinputfield.each_key do |key|
			 # the eval enables us to use the accesor, without having to change the attr hash directly (which is bad!)
			 eval("$currentedit.data[layer].#{key} = getHexSequences(pktinputfield[key].text)") if $currentedit.data[layer].attr[key].to_s != pktinputfield[key].text
			end
		end
	else	
		# raw data just put a textbox to enter the string
		frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE|PACK_UNIFORM_WIDTH)
		FXLabel.new(frame,"Raw data")
		data = FXTextField.new(frame,50)
		data.text = $currentedit.data[layer]
		
		exitcode = dlg.execute
		if exitcode != 0
			$currentedit.data[layer] = getHexSequences(data.text)
		end
	end
	updateTree()
end
end

# Function to translate the \x?? sequences in strings
def getHexSequences(string)
	string.gsub(/\\x([A-Fa-f0-9]*)/){ |m| sprintf("%c", m.gsub(/\\x/,'').hex)}
end
def pktCode(pkt)
	str = ""
	pkt.each do |layer|
		str << layer.to_code(true) + "+"if layer.kind_of? Pkt
		str << "\"" + layer + "\"+" if layer.kind_of? String
	end
	str.chomp!("+") # get rid of the last +
	return str
end	

def showCode(code)
	dlg=FXDialogBox.new($main,"Ruby Code",LAYOUT_FIX_WIDTH|DECOR_TITLE|DECOR_BORDER,0,0,300)
	label = FXText.new(dlg,nil,0,TEXT_READONLY|LAYOUT_FILL_X)
	label.text = code
	accept = FXButton.new(dlg, "&OK", nil, dlg, FXDialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X|LAYOUT_CENTER_Y)
	accept.setDefault  
    	accept.setFocus
	dlg.execute
end

def showDlg(title,string)
	dlg=FXDialogBox.new($main,title)
	FXLabel.new(dlg,string,nil,JUSTIFY_LEFT)
	accept = FXButton.new(dlg, "&OK", nil, dlg, FXDialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X|LAYOUT_CENTER_Y)
	accept.setDefault  
    	accept.setFocus
	dlg.execute
end

def showError(error)
	showDlg("ERROR",error)
end


# draw an editable packet (list of layers) on the edit tab
def layersDraw(layers)
	1.upto(5) do |layer|
		if layers[layer].kind_of? Pkt
			proto = layers[layer].class.to_s.gsub("Pkt","").upcase	
			$layers[layer].icon = $protoicon[proto]
		else
			if layer.kind_of? String
				$layers[layer].icon = $protoicon["Raw data"]
			else
				$layers[layer].icon = $layericon[layer]
			end
		end
	end
end

def addPkt(pkt,name=nil)
	list = [nil,nil,nil,nil,nil]
	foo = pkt.dup
	1.upto(5)do |layer|
		if foo.getlayer(layer) != false
			bar = foo.getlayer(layer).dup
			bar.head!
			list[layer] = bar
		end
	end
	$count = $count + 1
	name = "Packet#{$count}" if name == nil
	tmp = $pktTree.appendItem(nil, name)
	tmp.data = list
	selectPkt(tmp)
	updateTree
end

# select a packet in the list and work with it
def selectPkt(item)
	if item.parent == nil #it is a packet
		$currentedit = item
		$title.text = item.text
		layersDraw(item.data)
	end
end

def pktRename
	# put a textbox to enter the string
	dlg=FXDialogBox.new($main,"Rename Packet")
	# Bottom buttons
	buttons = FXHorizontalFrame.new(dlg, LAYOUT_SIDE_BOTTOM|FRAME_NONE|LAYOUT_FILL_X,0, 0, 0, 0, 40, 40, 20, 20)
	
	# Accept
	accept = FXButton.new(buttons, "&Accept", nil, dlg, FXDialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
 	accept.setDefault  
    	accept.setFocus   
	
	# Cancel
	FXButton.new(buttons, "&Cancel", nil, dlg, FXDialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
 
	frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE)
	FXLabel.new(frame,"New Name :")
	data = FXTextField.new(frame,50)
	exitcode = dlg.execute
	if exitcode != 0
		$currentedit.text = correctName(data.text)
	end
end

# A packet name should not have any "bad char"
def correctName(name)
	name.gsub!(/\W/,'_')
	return name
end

def pktDelete
	if getNumPkt < 1
		# should NEVER happen
		puts "Pkt count dropped below 1, impossible !"
		$count = 0
		addPkt(NullPkt.new)
		return
	end
	if getNumPkt == 1
		# we want to have at least one packet
		$pktTree.removeItem($currentedit)
		$count = 0
		addPkt(NullPkt.new)
	else
# TODO : find a better way to do this packet selection, without having to play with coordinates !
		tmp = $pktTree.getItemAt(0,0) # the first packet in the list
		tmp = $pktTree.getItemAt(0,25) if tmp == $currentedit # or the second if need be
		$pktTree.removeItem($currentedit)
		selectPkt(tmp)
	end
end

def getNumPkt
	i = 0
	$pktTree.each { |a|
		i = i+1
	}
	return i
end

# new File
def newFile(mode=true)
	# get rid of all the packets
	$pktTree.clearItems
	$count = 0
	# depending on mode, we either create a new workspace with a blank packet, or leave the place empty
	addPkt(NullPkt.new) if mode
	
	$filename = nil
end

# Open a file
def openFile
	openDialog = FXFileDialog.new($main, "Open File")
	patterns = ["All Files (*)",
                    "Ruby File (*.rb)"
		   ]
	openDialog.patternList = patterns
	if openDialog.execute != 0
		if File.exists? openDialog.filename
			# clear the current workspace, we do not want a single packet remaining
			newFile(false)
			# open file, discard comment lines	
			$filename = openDialog.filename
			file = File.new(openDialog.filename,"r")
			file.each do |line|
				if !(line =~ /#/)
					# get the name, which is like name=Pkt(...)
					name = correctName(line.slice!(0,line.index("=")+1).chomp!("="))
					p = eval(line)
					addPkt(p,name) if p.kind_of? Pkt
				end
			end
			if $count < 0
				addPkt(NullPkt.new)
			end
			$pktTree.each do |pkt|
				$currentedit = pkt
				updateTree
			end
		end
    	end
end

# Save a file, writing ruby code directly (can be easily included and reused)
def saveFile
     	if $filename
		file = File.new($filename,"w")
     		file.puts "# File generated by VisualForge, do not edit "
		$pktTree.each do |pkt|
			file.print correctName(pkt.text),"=",pktCode(pkt.data),"\n"
		end
		file.close
	else
		saveFileAs
	end	
end

def saveFileAs
	saveDialog = FXFileDialog.new($main, "Save File")
	if saveDialog.execute != 0
		if File.exists? saveDialog.filename
			if MBOX_CLICKED_NO == FXMessageBox.question($main, MBOX_YES_NO,"Overwrite File", "Overwrite existing file?")
				return 
			end
		end
	
	$filename = saveDialog.filename
	saveFile
	end
end

def preferences
# put a textbox to enter the string
	dlg=FXDialogBox.new($main,"Settings")
	# Bottom buttons
	buttons = FXHorizontalFrame.new(dlg, LAYOUT_SIDE_BOTTOM|FRAME_NONE|LAYOUT_FILL_X,0, 0, 0, 0, 40, 40, 20, 20)
	
	# Accept
	accept = FXButton.new(buttons, "&Accept", nil, dlg, FXDialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
 	accept.setDefault  
    	accept.setFocus   
	
	# Cancel
	FXButton.new(buttons, "&Cancel", nil, dlg, FXDialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
 
	frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE)
	FXLabel.new(frame,"Interface :")
	inter = FXTextField.new(frame,6)
	inter.text = "lo"
	frame=FXHorizontalFrame.new(dlg, LAYOUT_SIDE_TOP|FRAME_NONE)
	FXLabel.new(frame,"Timeout for receiving packets (seconds) :")
	timeout = FXTextField.new(frame,6)
	timeout.text = "15"
	exitcode = dlg.execute
	if exitcode != 0
		begin
			init(inter.text)
		rescue
			showError("BAD INTERFACE")
		end
		$TIMEOUT = timeout.text.to_i
	end

end

# add a packet to start with
addPkt(NullPkt.new)

FXToolTip.new($app)
$app.create
$main.show()
$app.run
